/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


#include <iostream.h>
#include <signal.h>
#include <stdio.h>

#include "sockrpc.hxx"
#include "pollset.hxx"
#include "struct_test.hxx"

#include <sys/types.h>
#include <sys/time.h>
#include <sys/timeb.h>

struct timeval Tbegin;
struct timeval Tend;
struct timeval tt ;

void timeBegin()
{
  gettimeofday(&Tbegin, NULL);
}

void timeEnd()
{
  gettimeofday(&Tend, NULL);
}

double timeDiff()
{
  double res = (Tend.tv_sec - Tbegin.tv_sec) ;
  res += ((float)(Tend.tv_usec)/1000000) - ((float)(Tbegin.tv_usec)/1000000) ;
  return res ;
}

#include "class_clnt.hxx"

void lineBroken(SvcSocket* s) {
  char name[256] ;
  cout << "Le serveur sur " << s->tohost(name) << " est tombe " << endl ;
  exit(1) ;
}

int main(int argc, char *argv[])
{
  if( argc < 2) {
    cerr << "error: usage : " << argv[0] << "<filter>\n";
    exit(1);
  }

  SvcInit() ;
  proxy(MonObjServ) svc; 
  int     n;
  char tohost[256] ;

  svc.linkBreak(lineBroken) ;
  if (svc.import(argv[1]) == -1) {
    cerr << "Error during connection" << endl ;
    exit(1) ;
  }
  cerr << "connected to " << svc.tohost(tohost) << " " 
       << svc.toport() << "\n";

  cout << "Command ? " ;
  cin >> n ;

  char *serv, *host, *filter ;
  int port = 0 ;
  int nbinst = 0 ;

  while (n != 0) {
    switch (n) {
    case 1 :
      serv = new char[256] ;
      filter = new char[256] ;
      host = new char[256] ;
      cout << "Appel a publish" << endl ;
      cout << "Service ? " ;
      cin >> serv ;
      cout << endl << "filter ? " ;
      cin >> filter ;
      cout << endl << "Port ? " ;
      cin >> port ;
      cout << endl << "Nombre d'instances ? " ;
      cin >> nbinst ;
      cout << serv << filter << port << nbinst << endl ;
      int res = svc.publish(serv, filter, port, nbinst) ;
      if (res != nbinst) {
	printf("Erreur de transfert\n") ;
	exit(1) ;
      }
      delete serv ; delete filter ; delete host ;
      cout << endl ;
      break ;
    case 2 :
      serv = new char[256] ;
      filter = new char[256] ;
      cout << "Appel a subscript" << endl ;
      cout << "Service ? " ;
      cin >> serv ;
      cout << endl << "filter ? " ;
      cin >> filter ;
      host = svc.subscript(serv, filter, port) ;
      cout << endl << "Host : " << host ;
      cout << endl << "Port : " << port ;
      cout << endl ;
      delete serv ; delete filter ; delete host ;
      break ;
    case 3 :
      cout << "Appel a bench" << endl ;
      cout << "Nombre d'iteration ? " ;
      cin >> nbinst ;
      cout << endl ;
      Page p ;
      OID oid ;
      oid.numobj = 127 ;
      timeBegin() ;
      int i ;
      for (i = 0; i < nbinst; i++) {       
	svc.bench(oid, p) ;
	if ( (p.tab[0] != 10) ||
	     (p.tab[1] != 20) ||
	     (p.tab[2] != 30) ||
	     (p.tab[4093] != 40) ||
	     (p.tab[4094] != 50) ||
	     (p.tab[4095] != 60)) {
	  printf("Erreur de transfert\n") ;
	  exit(1) ;
	}
      }
      timeEnd() ;
      printf("Temps ecoule : %f\n", timeDiff()) ;
      break ;
    default :
      break ;
    }
    cout << "Command ? " ;
    cin >> n ;
  }
}
